<?php

namespace Drp\UserManagement\Test\Feature;


use Carbon\Carbon;
use Drp\UserManagement\Contracts\HasRoles;
use Drp\UserManagement\Events\AccountActivated;
use Drp\UserManagement\Test\TestCase;
use Drp\UserManagement\User;
use Illuminate\Database\Eloquent\ModelNotFoundException;


class UserModelTest extends TestCase
{
    /**
     * @test
     */
    public function it_returns_only_users_with_a_matching_role()
    {
        factory(User::class)->create(['role' => 'Super Administrator']);
        factory(User::class)->create(['role' => 'Administrator']);

        $results = app(User::class)->usersByRoles('Super Administrator');

        $this->assertCount(1, $results);

        $results = app(User::class)->usersByRoles([
            'Super Administrator',
            'Administrator'
        ]);

        $this->assertCount(2, $results);
    }

    /**
     * @test
     */
    public function it_returns_only_super_admin_users()
    {
        $superAdminUser = factory(User::class)->create(['role' => 'Super Administrator']);
        $adminUser = factory(User::class)->create(['role' => 'Administrator']);
        $user = factory(User::class)->create(['role' => 'User']);

        $results = app(User::class)->allSuperAdmins();
        $this->assertCount(1, $results);
        $this->assertContainsModel($superAdminUser, $results);
        $this->assertNotContainsModel($adminUser, $results);
        $this->assertNotContainsModel($user, $results);
    }

    /**
     * @test
     */
    public function it_returns_only_admin_users()
    {
        $superAdminUser = factory(User::class)->create(['role' => 'Super Administrator']);
        $adminUser = factory(User::class)->create(['role' => 'Administrator']);
        $user = factory(User::class)->create(['role' => 'User']);

        $results = app(User::class)->allAdmins();
        $this->assertCount(2, $results);
        $this->assertContainsModel($superAdminUser, $results);
        $this->assertContainsModel($adminUser, $results);
        $this->assertNotContainsModel($user, $results);
    }

    /**
     * @test
     */
    public function it_determines_if_the_user_has_super_admin_access()
    {
        $superAdminUser = factory(User::class)->make(['role' => 'Super Administrator']);
        $adminUser = factory(User::class)->make(['role' => 'Administrator']);
        $user = factory(User::class)->make(['role' => 'User']);

        $this->assertTrue($superAdminUser->hasSuperAdminAccess());
        $this->assertFalse($adminUser->hasSuperAdminAccess());
        $this->assertFalse($user->hasSuperAdminAccess());
    }

    /**
     * @test
     */
    public function it_determines_if_the_user_has_admin_access()
    {
        $superAdminUser = factory(User::class)->make(['role' => 'Super Administrator']);
        $adminUser = factory(User::class)->make(['role' => 'Administrator']);
        $user = factory(User::class)->make(['role' => 'User']);

        $this->assertTrue($superAdminUser->hasAdminAccess());
        $this->assertTrue($adminUser->hasAdminAccess());
        $this->assertFalse($user->hasAdminAccess());
    }

    /**
     * @test
     */
    public function it_determines_if_a_user_is_a_specific_role()
    {
        $adminUser = factory(User::class)->make(['role' => 'Super Administrator']);

        $this->assertTrue($adminUser->hasRole('Super Administrator'));
        $this->assertFalse($adminUser->hasRole('User'));
    }

    /**
     * @test
     */
    public function it_returns_the_user_with_matching_login_token()
    {
        $user1 = factory(User::class)->create(['first_login_token' => 'TOKEN']);
        $user2 = factory(User::class)->create(['first_login_token' => 'TOKEN2']);

        $results = app(User::class)->getByFirstLoginToken('TOKEN');

        $this->assertSameModel($user1, $results);
        $this->assertNotSameModel($user2, $results);

        $this->expectException(ModelNotFoundException::class);
        app(User::class)->getByFirstLoginToken('INVALID_TOKEN');
    }

    /**
     * @test
     */
    public function it_can_reset_the_login_token()
    {
        $user = factory(User::class)->create(['first_login_token' => 'TOKEN']);

        $this->assertEquals('TOKEN', $user->first_login_token);
        $user->removeFirstLoginToken();
        $this->assertNull($user->fresh()->first_login_token);
    }

    /**
     * @test
     */
    public function it_can_add_a_login_token()
    {
        $user = factory(User::class)->create();
        $expiryDate = Carbon::create(2017, 01, 01);
        $this->assertNull($user->first_login_token);

        $user->addLoginToken('TOKEN', $expiryDate);

        $this->assertEquals('TOKEN', $user->fresh()->loginToken());
        $this->assertEquals($expiryDate, $user->fresh()->loginExpiry());
    }

    /**
     * @test
     */
    public function it_determines_if_the_user_requires_approving()
    {
        $user = factory(User::class)->make(['approved' => true]);
        $this->assertFalse($user->requiresApproving());

        $user = factory(User::class)->make(['approved' => false]);
        $this->assertTrue($user->requiresApproving());
    }

    /**
     * @test
     */
    public function it_returns_only_users_awaiting_approval()
    {
        $approvedUser = factory(User::class)->create(['approved' => true]);
        $unApprovedUser = factory(User::class)->create(['approved' => false]);

        $results = app(User::class)->allAwaitingApproval();
        $this->assertCount(1, $results);
        $this->assertContainsModel($unApprovedUser, $results);
        $this->assertNotContainsModel($approvedUser, $results);
    }

    /**
     * @test
     */
    public function it_can_approve_a_user()
    {
        $this->expectsEvents(AccountActivated::class);
        $user = factory(User::class)->create(['approved' => false]);

        $this->assertFalse($user->approved);
        $user->approve();
        $this->assertTrue($user->fresh()->approved);
    }
}
